<?php

namespace Drupal\monarch_global_css\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class BulkUploadForm.
 */
class CssGlobalEditor extends FormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->fileSystem = $container->get('file_system');
    $instance->state = $container->get('state');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'css_global_editor_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['global_css'] = [
      '#type' => 'textarea',
      '#title' => 'Global CSS',
      '#default_value' => @file_get_contents('public://global.css') ?: '' ?? '',
      '#rows' => '15',
      '#cols' => '60',
      '#resisable' => TRUE,
      // 100 MB
      '#maxlength' => 104857600,
      '#attributes' => [
        'class' => [
          'codemirror-custom',
          'codemirror-css',
        ],
      ],
      '#attached' => [
        'library' => [
          'monarch_global_css/codemirror.css',
        ],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data = $form_state->getValue('global_css') ?: '';
    $hash = @md5($data);

    if ($hash !== @md5_file('public://global.css')) {
      file_put_contents('public://global.css', $data);
      @$this->fileSystem->copy('public://global.css', implode('.', array_filter([
        'public://global',
        date('Y-m-d\THis'),
        $hash,
        'css',
      ])), FileSystemInterface::EXISTS_REPLACE);
      $fs = $this->entityTypeManager->getStorage('file');
      /** @var \Drupal\file\FileInterface $file */
      $file = current($fs->loadByProperties([
        'uri' => 'public://global.css',
      ])) ?: $fs->create([
        'filename' => 'global.css',
        'uri' => 'public://global.css',
      ]);

      $file->setPermanent();
      $file->save();
      drupal_flush_all_caches();
    }
  }

}
